@@ -1,8 +1,6 @@
 """The Jupyter HTML Notebook"""
 
 import os
-# Packagers: modify this line if you store the notebook static files elsewhere
-DEFAULT_STATIC_FILES_PATH = os.path.join(os.path.dirname(__file__), "static")
 
 # Packagers: modify the next line if you store the notebook template files
 # elsewhere
